import SwiftUI

import AppIntents

let accounts = [
    AccountEntity(id: "1", name: "たろう"),
    AccountEntity(id: "2", name: "じろう"),
    AccountEntity(id: "3", name: "さぶろう"),
    AccountEntity(id: "4", name: "しろう"),
    AccountEntity(id: "5", name: "ごろう"),
]

struct AccountQuery: EntityQuery {
    func entities(for identifiers: [AccountEntity.ID]) async throws -> [AccountEntity] {
        return accounts.filter { entity in
            identifiers.contains(entity.id)
        }
    }

    func suggestedEntities() async throws -> [AccountEntity] {
        return accounts
    }
}

struct AccountEntity: AppEntity, Identifiable {
    static var defaultQuery: AccountQuery = AccountQuery()

    let id: String
    let name: String

    init(id: String, name: String) {
        self.id = id
        self.name = name
    }

    var displayRepresentation: DisplayRepresentation {
        DisplayRepresentation(title: "\(name)")
    }

    static var typeDisplayRepresentation: TypeDisplayRepresentation = "Account"
}


struct ExampleFocusFilter: SetFocusFilterIntent {
    static var title: LocalizedStringResource = "Example"

    var displayRepresentation: DisplayRepresentation {
        if let account {
            return DisplayRepresentation(title: "\(account.name)")
        } else {
            return ""
        }
    }

    @Parameter(title: "Selected Account")
    var account: AccountEntity?

    func perform() async throws -> some IntentResult {
        let defaults = UserDefaults.standard
        let key = "selectedAccount"
        if let account {
            defaults.set(account.id, forKey: key)
        } else {
            defaults.removeObject(forKey: key)
        }
        return .result()
    }
}

func loadCurrentFocusState() async {
    let defaults = UserDefaults.standard
    let key = "selectedAccount"
    do {
        let current = try await ExampleFocusFilter.current
        if let account = current.account {
            defaults.set(account.id, forKey: key)
        } else {
            defaults.removeObject(forKey: key)
        }
    } catch {
        defaults.removeObject(forKey: key)
    }
}

struct FocusFilterExample: View {
    @AppStorage("selectedAccount") var selectedAccountID: String?

    var body: some View {
        LabeledContent("Selected Account") {
            if let selectedAccountID, let account = accounts.filter { $0.id == selectedAccountID }.first {
                Text(account.name)
            }
        }
        .padding()
        .task {
            await loadCurrentFocusState()
        }
    }
}

struct FocusFilterExample_Previews: PreviewProvider {
    static var previews: some View {
        FocusFilterExample()
    }
}
