import SwiftUI

import Algorithms

struct EqualWidthHStack: Layout {
    private func maximumSize(_ subviews: Subviews, proposal: ProposedViewSize = .unspecified) -> (CGSize, ProposedViewSize) {
        let subviewProposal = ProposedViewSize(
            width: proposal.width.map { $0 / CGFloat(subviews.count) },
            height: proposal.height
        )
        let width = subviews.map {
            $0.dimensions(in: subviewProposal).width
        }.max() ?? 0
        let height = subviews.map {
            $0.dimensions(in: subviewProposal).height
        }.max() ?? 0
        return (CGSize(width: width, height: height), subviewProposal)
    }

    func sizeThatFits(proposal: ProposedViewSize, subviews: Subviews, cache: inout LayoutCache) -> CGSize {
        let spacing = cache.spacings
        let totalSpacing = cache.totalSpacing
        let (max, _) = maximumSize(
            subviews,
            proposal: ProposedViewSize(
                width: proposal.width.map { $0 - totalSpacing },
                height: proposal.height
            )
        )

        return CGSize(
            width: max.width * CGFloat(subviews.count) + spacing.reduce(0, +),
            height: max.height
        )
    }

    func placeSubviews(in bounds: CGRect, proposal: ProposedViewSize, subviews: Subviews, cache: inout LayoutCache) {
        let spacing = cache.spacings
        let totalSpacing = cache.totalSpacing
        let (max, subviewProposal) = maximumSize(
            subviews,
            proposal: ProposedViewSize(
                width: proposal.width.map { $0 - totalSpacing },
                height: proposal.height
            )
        )

        for (i, subview) in subviews.enumerated() {
            let x = bounds.minX +
            max.width * CGFloat(i) +
            max.width / 2 +
            spacing.prefix(i).reduce(0, +)
            let y = bounds.midY
            subview.place(
                at: CGPoint(x: x, y: y),
                anchor: .center,
                proposal: subviewProposal
            )
        }
    }
}

extension EqualWidthHStack {
    private func spacings(_ subviews: Subviews) -> [CGFloat] {
        return subviews.adjacentPairs().map { (subview1, subview2) in
            subview1.spacing.distance(to: subview2.spacing, along: .horizontal)
        }
    }

    struct LayoutCache {
        let spacings: [CGFloat]
        let totalSpacing: CGFloat
    }

    typealias Cache = LayoutCache

    func makeCache(subviews: Subviews) -> LayoutCache {
        let spacing = spacings(subviews)
        let totalSpacing = spacing.reduce(0, +)
        return LayoutCache(spacings: spacing, totalSpacing: totalSpacing)
    }
}

struct CustomLayoutExample: View {
    var body: some View {
        EqualWidthHStack {
            Text("Hello")
                .background(Color.blue)
            Text("Hello World")
                .background(Color.yellow)
            Text("Hello World!!")
                .background(Color.red)
        }
        .frame(minWidth: 100, maxWidth: 320)
        .background(Color.gray)
    }
}

struct CustomLayoutExample_Previews: PreviewProvider {
    static var previews: some View {
        CustomLayoutExample()
    }
}
