import SwiftUI
import PhotosUI

import ImageIO

extension CGImagePropertyOrientation {
    var imageOrientation: Image.Orientation {
        switch self {
        case .up:
            return .up
        case .upMirrored:
            return .upMirrored
        case .down:
            return .down
        case .downMirrored:
            return .downMirrored
        case .leftMirrored:
            return .leftMirrored
        case .right:
            return .right
        case .rightMirrored:
            return .rightMirrored
        case .left:
            return .left
        }
    }
}

struct ImageImporter: Transferable {
    let image: Image

    enum Error: Swift.Error {
        case invalidData
    }

    static var transferRepresentation: some TransferRepresentation {
        DataRepresentation(importedContentType: .image) { data in
            guard let source = CGImageSourceCreateWithData(data as CFData, nil) else {
                throw Error.invalidData
            }
            let index = CGImageSourceGetPrimaryImageIndex(source)
            guard let cgImage = CGImageSourceCreateImageAtIndex(source, index, nil) else {
                throw Error.invalidData
            }
            guard let properties = CGImageSourceCopyPropertiesAtIndex(source, index, nil) else {
                throw Error.invalidData
            }
            let orientation = ((properties as NSDictionary)[kCGImagePropertyOrientation as String] as? UInt32).flatMap(CGImagePropertyOrientation.init(rawValue:))
            return ImageImporter(image: Image(decorative: cgImage, scale: 1, orientation: orientation?.imageOrientation ?? .up))
        }
    }
}

struct PhotosPickerExample: View {
    @State private var item: PhotosPickerItem? = nil
    @State private var image: Image? = nil

    var body: some View {
        VStack {
            if let image {
                image
                    .resizable()
                    .aspectRatio(contentMode: .fit)
            }
            PhotosPicker("Photos", selection: $item)
        }
        .onChange(of: item) { item in
            Task {
                guard let item else {
                    image = nil
                    return
                }
                guard let importer = try await item.loadTransferable(type: ImageImporter.self) else {
                    return
                }
                image = importer.image
            }
        }
    }
}

struct PhotoPickerExample_Previews: PreviewProvider {
    static var previews: some View {
        PhotosPickerExample()
    }
}
