import SwiftUI
import CoreTransferable
import UniformTypeIdentifiers

import HTMLEntities

struct Anchor: Codable, Hashable {
    enum Error: Swift.Error {
        case invalidHTML
        case incompatibleData
    }

    let text: String
    let href: URL

    init(text: String, href: URL) {
        self.text = text
        self.href = href
    }

    init(html: String) throws {
        guard let match = try #/<a href="([^<]*)">([^<]*)</a>/#.wholeMatch(in: html) else {
            throw Error.invalidHTML
        }
        let (_, href, text) = match.output
        guard let href = URL(string: String(href)) else {
            throw Error.invalidHTML
        }
        self.init(text: String(text), href: href)
    }

    var html: String {
        return "<a href=\"\(href.absoluteString.htmlEscape())\">\(text.htmlEscape())</a>"
    }
}

extension UTType {
    static var anchor: UTType { UTType(exportedAs: "com.example.anchor") }
}

extension Anchor: Transferable {
    static var transferRepresentation: some TransferRepresentation {
        CodableRepresentation(contentType: .anchor)
        DataRepresentation(contentType: .html) { anchor in
            return anchor.html.data(using: .utf8)!
        } importing: { data in
            guard let html = String(data: data, encoding: .utf8) else {
                throw Error.incompatibleData
            }
            return try Anchor(html: html)
        }
        ProxyRepresentation(exporting: \.href)
        ProxyRepresentation(exporting: \.text)
    }
}

extension Anchor: Identifiable {
    var id: String { html }
}

let initialAnchors: [Anchor] = [
    Anchor(text: "Apple", href: URL(string: "https://www.apple.com")!),
    Anchor(text: "Google", href: URL(string: "https://www.google.com")!),
]

struct AnchorList: View {
    @State var anchors: [Anchor] = []

    var body: some View {
        VStack {
            List {
                Section(header: Text("Source")) {
                    ForEach(initialAnchors) { anchor in
                        Text(anchor.text)
                            .contextMenu {
                                Button("Copy") {
                                    UIPasteboard.general.setValue(anchor.html, forPasteboardType: UTType.html.identifier)
                                }
                            }
                            .draggable(anchor)
                    }
                }
                Section(header: Text("Destination")) {
                    ForEach(anchors) { anchor in
                        NavigationLink(anchor.text, value: anchor)
                    }
                }
            }

            HStack {
                Text("Drop here")
                    .dropDestination(for: Anchor.self) { items, location in
                        anchors.append(contentsOf: items)
                        return true
                    }
                Spacer()
                PasteButton(payloadType: Anchor.self) { items in
                    anchors.append(contentsOf: items)
                }
            }
            .padding()
        }
    }
}

struct AnchorView: View {
    let anchor: Anchor

    var body: some View {
        VStack {
            Link(anchor.text, destination: anchor.href)
            ShareLink(
                item: anchor.href,
                subject: Text(anchor.href.absoluteString),
                message: Text(anchor.text)
            )
        }
    }
}

struct TransferableExample: View {
    var body: some View {
        AnchorList()
            .navigationDestination(for: Anchor.self) { anchor in
                AnchorView(anchor: anchor)
            }
    }
}

struct TransferableExample_Previews: PreviewProvider {
    static var previews: some View {
        TransferableExample()
    }
}
