import SwiftUI

struct DirectoryView: View {
    @StateObject private var model: DirectoryModel
    @Binding private var selectedFile: File?
    private var selectedItem: Binding<FileSystemItem?> {
        Binding {
            if let selectedFile {
                return .file(selectedFile)
            } else {
                return nil
            }
        } set: { item in
            switch item {
            case .file(let file):
                selectedFile = file
            default:
                selectedFile = nil
            }
        }

    }

    init(_ directory: Directory, selectedFile: Binding<File?>) {
        self._model = StateObject(wrappedValue: DirectoryModel(directory: directory))
        self._selectedFile = selectedFile
    }

    var body: some View {
        List(selection: selectedItem) {
            ForEach(model.items) { item in
                switch item {
                case .file(let file):
                    Label(file.url.lastPathComponent, systemImage: "doc")
                        .tag(item)
                case .directory(let directory):
                    NavigationLink(value: directory) {
                        Label(directory.url.lastPathComponent, systemImage: "folder")
                    }
                }
            }
            .onDelete { indexSet in
                for item in indexSet.map({ model.items[$0] }) {
                    switch item {
                    case .directory(let directory):
                        try? model.removeDirectory(directory)
                    case .file(let file):
                        try? model.removeFile(file)
                    }
                }
            }
        }
        .onAppear {
            try? model.load()
        }
        .navigationTitle(model.name)
        .toolbar {
            ToolbarItemGroup(placement: .primaryAction) {
                Button {
                    _ = try? model.createDirectory(name: UUID().uuidString)
                } label: {
                    Label("Create Directory", systemImage: "folder.badge.plus")
                }
                Button {
                    _ = model.createFile(name: UUID().uuidString + ".txt")
                } label: {
                    Label("Create File", systemImage: "doc.badge.plus")
                }

            }
        }
    }
}

struct DirectoryView_Previews: PreviewProvider {
    static var previews: some View {
        NavigationStack {
            DirectoryView(
                Directory(url: try! FileManager.default.url(for: .documentDirectory, in: .userDomainMask, appropriateFor: nil, create: true)),
                selectedFile: .constant(nil)
            )
        }
    }
}
