//
//  Intent.swift
//  AppIntentExample
//
//  Created by Kouki Saito on 2022/10/23.
//

import SwiftUI
import AppIntents

struct FirstShortcutsProvider: AppShortcutsProvider {
    static var appShortcuts: [AppShortcut] {
        AppShortcut(
            intent: FirstIntent(),
            phrases: [
                "\(.applicationName)を始める",
            ]
        )
    }
}

struct FirstIntent: AppIntent {
    static var title:
      LocalizedStringResource = "最初のインテント"

    @Parameter(title: "入力")
    var input: String

    func perform() async throws -> some IntentResult {
        return .result(dialog: "入力は\(input)")
    }
}

struct OpenMyBookIntent: AppIntent {
    static var title: LocalizedStringResource = "本を開く"

    /*
     // AppEntity対応前
     @Parameter(title: "本のタイトル")
     var title: String
     func perform() async throws -> some IntentResult {
         return .result(dialog: "本のタイトルは\(title)")
     }
     */

    @Parameter(title: "本")
    var book: Book

    func perform() async throws -> some IntentResult {
        return .result(dialog: "本のタイトルは\(book.title)")
    }

    static var parameterSummary: some ParameterSummary {
        Summary("\(\.$book)を開く")
    }
}



enum BookCategory: String, AppEnum {
    case mystery
    case fantasy
    case nonFiction

    static var typeDisplayRepresentation:
      TypeDisplayRepresentation = "カテゴリ"
    static var caseDisplayRepresentations:
      [BookCategory : DisplayRepresentation] = [
        .mystery: "ミステリ",
        .fantasy: "ファンタジー",
        .nonFiction: "ノンフィクション",
      ]
}

struct Book: AppEntity {
    var id: String

    @Property(title: "タイトル")
    var title: String

    @Property(title: "URL")
    var url: String

    init(id: String, title: String) {
        self.id = id
        self.title = title
        self.url = "https://example.com/books/\(id)"
    }

    var displayRepresentation: DisplayRepresentation {
        DisplayRepresentation(title: "\(title)")
    }
    static var typeDisplayRepresentation: TypeDisplayRepresentation = "本"

    static var defaultQuery = BookQuery()
}

struct BookQuery: EntityStringQuery {
    func entities(for identifiers: [String]) async throws -> [Book] {
        return identifiers.compactMap { identifier in
            books.first(where: { $0.id == identifier })
        }
    }

    func suggestedEntities() async throws -> [Book] {
        books
    }

    func entities(matching string: String) async throws -> [Book] {
        return books.filter { $0.title.contains(string) }
    }
}



struct FindPopularBookIntent: AppIntent {
    static var title: LocalizedStringResource = "人気の本を探す"

    func perform() async throws ->
    some IntentResult & ReturnsValue<Book> {
        return .result(value: await Book.popularBook)
    }

    static var parameterSummary: some ParameterSummary {
        Summary("人気の本を探す")
    }
}



extension Book {
    static var popularBook: Book {
        get async {
            books.randomElement()!
        }
    }
}

let books = [
    Book(id: "1", title: "1つ目の本"),
    Book(id: "2", title: "2つ目の本"),
    Book(id: "3", title: "3つ目の本"),
]
