//
//  MyWidget.swift
//  MyWidget
//
//  Created by Kouki Saito on 2022/10/23.
//

import WidgetKit
import SwiftUI

struct Provider: TimelineProvider {
    func placeholder(in context: Context) -> SimpleEntry {
        SimpleEntry(date: Date())
    }

    func getSnapshot(in context: Context, completion: @escaping (SimpleEntry) -> ()) {
        let entry = SimpleEntry(date: Date())
        completion(entry)
    }

    func getTimeline(in context: Context, completion: @escaping (Timeline<Entry>) -> ()) {
        var entries: [SimpleEntry] = []

        // Generate a timeline consisting of five entries an hour apart, starting from the current date.
        let currentDate = Date()
        for hourOffset in 0 ..< 5 {
            let entryDate = Calendar.current.date(byAdding: .hour, value: hourOffset, to: currentDate)!
            let entry = SimpleEntry(date: entryDate)
            entries.append(entry)
        }

        let timeline = Timeline(entries: entries, policy: .atEnd)
        completion(timeline)
    }
}

struct SimpleEntry: TimelineEntry {
    let date: Date
}

struct WidgetEntryView : View {
    var entry: Provider.Entry
    @Environment(\.widgetFamily)
    var widgetFamily

    var body: some View {
        switch widgetFamily {
        case .accessoryRectangular:
            VStack(alignment: .leading) {
                Text("\(Image(systemName: "iphone")) iPhone")
                    .font(.headline)
                Text("iPhone 14 Pro")
                ProgressView(value: 0.2)
            }
            .frame(maxWidth: .infinity, alignment: .leading)
        case .accessoryCircular:
            ProgressView(value: 0.2) {
                Image(systemName: "iphone")
            }
            .progressViewStyle(.circular)
        case .accessoryInline:
            Text("\(Image(systemName: "iphone")) iPhone")
        default:
            ViewThatFits {
                Text("Hello, Widget!")
                Text("Hello\nWidget!")
                    .multilineTextAlignment(.center)
            }
        }
    }
}

@main
struct StaticWidget: Widget {
    let kind: String = "StaticWidget"

    var body: some WidgetConfiguration {
        StaticConfiguration(
            kind: kind,
            provider: Provider(),
            content:  { entry in
                WidgetEntryView(entry: entry)
            }
        )
        .configurationDisplayName("Static Widget")
        .description("説明")
        .supportedFamilies([
            .accessoryCircular,
            .accessoryInline,
            .accessoryRectangular,
        ])
    }
}

struct MyWidget_Previews: PreviewProvider {
    static var previews: some View {
        WidgetEntryView(entry: SimpleEntry(date: Date()))
            .previewContext(WidgetPreviewContext(family: .systemSmall))
    }
}
